-- 冒险礼包(章节礼包)
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local rechargemod = require "model.rechargemod"
local asset = require "model.asset"
local payment = require "model.payment"
local reward = require "model.reward"
local util = require "util"
local activitytimemod

local module_name = "adv_gift"
local module_id = MODULE_ID_ADV_GIFT
local _M = schema.new(module_name, {
    list = {},
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return {errno = STD_ERR.COMMON_ACT_NOT_OPEN} -- 活动未开启
    end
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

local function get_conf(character)
    return activitytimemod.get_conf(character, module_id)
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("recharge.money",function(_, cfid, orderid, moneynum, giftid, id, pos)
        if module_id == id then
            THIS.gift_buy(character, giftid)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    activitytimemod = require "model.activitytimemod"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
end

function MODULE.open(character, ainfo)
end

function MODULE.close(character, ainfo)
end

function MODULE.continue_open(character, ainfo)
end


function THIS.send_nty(character)
    local d = _M.assert_get(character)
    local temp = {}
    for _, v in pairs(d.list) do
        table.insert(temp, v)
    end
    character.send("adv_gift_nty", {list = temp})
    return 0
end

function THIS.gift_buy(character, giftid)
    local d = _M.assert_get(character)
    d.list[giftid] = d.list[giftid] or {id = giftid, num = 0}
    d.list[giftid].num = d.list[giftid].num + 1
    _M.persist(character)
    THIS.send_nty(character)
end

function THIS.adv_gift_buy(character, args)
    local d = _M.assert_get(character)
    local conf = get_conf(character)

    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local bfind = false
    for i = 1, #conf.gift do
        if conf.gift[i] == id then
            bfind = true
            break
        end
    end

    if not bfind then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if d.list[id] and d.list[id].num >= 1 then
        return STD_ERR.COMMON_HAVE_GIFT -- 已购买
    end
    
    local gift_conf = asset.GiftConfig_proto[id]
    if not gift_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if gift_conf.type ~= MODULE_ID_ADV_GIFT then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local pay_list = {}
    if gift_conf.spendtype == PAY_TYPE_FREE then
        pay_list = nil
    elseif gift_conf.spendtype == PAY_TYPE_ITEM then 
        table.insert(pay_list, {id = gift_conf.spend, num = gift_conf.num})
    elseif gift_conf.spendtype == PAY_TYPE_GIFT then
        rechargemod.get_orderid(character, gift_conf.ID)
        return 0
    end

    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module=module_name,
            brief="购买冒险礼包",
            context= "购买冒险礼包",
            mailgen={subject=2, body=""},
            notify={
                flags="adv_gift_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end
    THIS.gift_buy(character, id)
    return 0
end

function REQUEST.adv_gift_data(character, args)
    return func_ret("send_nty", character, args)
end

function REQUEST.adv_gift_buy(character, args)
    return func_ret("adv_gift_buy", character, args)
end

function MODULE.get_red_point(character)
    local d = _M.assert_get(character) or {}
    local gift_conf_list = asset.GiftConfig_proto
    for _, conf in pairs(gift_conf_list) do
        if conf.type == module_id then
            if not d.list[conf.ID] then
                return true, module_id
            end
        end
    end

end

return MODULE