-- 精英基金
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local rechargemod = require "model.rechargemod"
local asset = require "model.asset"
local payment = require "model.payment"
local reward = require "model.reward"
local util = require "util"
local activitytimemod
local adventure

local module_name = "fund3"
local module_id = MODULE_ID_FUND3
local func_type = 3

local _M = schema.new(module_name, {
    state = false,  -- 是否购买
    start_time = 0,
    free_award = 0,
    award = 0,
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return {errno = STD_ERR.COMMON_ACT_NOT_OPEN} -- 活动未开启
    end
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

local function get_conf(character)
    return activitytimemod.get_conf(character, module_id)
end

local function get_progress(character)
    adventure = adventure or require "model.adventure"
    return adventure.get_elite_pass(character)
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("recharge.money",function(_, cfid, orderid, moneynum, giftid, id, pos)
        if module_id == id then
            THIS.gift_unlock(character)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    activitytimemod = require "model.activitytimemod"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
end

function MODULE.open(character, ainfo)
end

function MODULE.close(character, ainfo)
end

function MODULE.continue_open(character, ainfo)
end

function THIS.fund_buy(character)
    local d = _M.assert_get(character)
    local conf = get_conf(character)
    if not conf.gift[1] then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    
    local gift_conf = asset.GiftConfig_proto[conf.gift[1]]
    if not gift_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if d.state then
        return STD_ERR.COMMON_HAVE_GIFT -- 已购买
    end

    local pay_list = {}
    if gift_conf.spendtype == PAY_TYPE_FREE then
        pay_list = nil
    elseif gift_conf.spendtype == PAY_TYPE_ITEM then 
        table.insert(pay_list, {id = gift_conf.spend, num = gift_conf.num})
    elseif gift_conf.spendtype == PAY_TYPE_GIFT then
        rechargemod.get_orderid(character, gift_conf.ID)
        return 0
    end
    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module=module_name,
            brief="购买精英基金",
            context= "购买精英基金",
            mailgen={subject=2, body=""},
            notify={
                flags="fund_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end
    THIS.gift_unlock(character)
    return 0
end

function THIS.fund_award(character, args)
    local d = _M.assert_get(character)

    local progress = get_progress(character)
    local max_award = d.free_award
    local award_list = {}
    for _, conf in pairs(asset.FundConfig_proto) do
        if conf.type == func_type and progress >= conf.condition then
            if conf.condition > d.free_award then
                max_award = math.max(max_award, conf.condition)
                for i = 1, #conf.free, 2 do
                    local item_id = conf.free[i]
                    local item_num = conf.free[i+1]
                    if item_num and item_num > 0 then
                        table.insert(award_list, {id = item_id, num = item_num})
                    else
                        break
                    end
                end
            end

            if d.state then
                if conf.condition > d.award then
                    for i = 1, #conf.Paid, 2 do
                        local item_id = conf.Paid[i]
                        local item_num = conf.Paid[i+1]
                        if item_num and item_num > 0 then
                            table.insert(award_list, {id = item_id, num = item_num})
                        else
                            break
                        end
                    end
                end
            end
        end 
    end

    if award_list and next(award_list) then
        local desc = {
            module="fund_award",
            brief="精英基金奖励",
            context= "精英基金奖励",
            mailgen={subject=2, body=""},
            notify={
                flags="fund_award"
            },
            detail=award_list
        }
        reward(character, desc)
    end
    d.free_award = max_award
    if d.state then
        d.award = max_award
    end
    _M.persist(character)
    return 0, {free_award = d.free_award, award = d.award, moduleid = module_id}
end

function THIS.gift_unlock(character)
    local d = _M.assert_get(character)
    if d.state then
        logger.warn("gift_unlock err", module_name)
        return
    end
    d.state = true
    d.start_time = util.today()+12*HOUR_SEC
    d.award = 0
    _M.persist(character)
    character.send("fund_nty", {
        moduleid = module_id, 
        state = d.state, 
        award = d.award,
        free_award = d.free_award,
        progress = get_progress(character)
    })
end

function THIS.get_client_data(character)
    local d = _M.assert_get(character)
    return 0, {
        moduleid = module_id,
        state = d.state,
        award = d.award,
        free_award = d.free_award,
        progress = get_progress(character)
    }
end

function MODULE.fund_buy(character)
    return func_ret("fund_buy", character)
end

function MODULE.get_client_data(character)
    return func_ret("get_client_data", character)
end

function MODULE.fund_award(character, args)
    return func_ret("fund_award", character, args)
end

function MODULE.check_over(character, args)
    return false
end

function MODULE.get_red_point(character)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return 
    end
    local d = _M.assert_get(character) or {}
    local progress = get_progress(character)
    for _, conf in pairs(asset.FundConfig_proto) do
        if conf.type == func_type and progress >= conf.condition and ((d.state and d.award < conf.condition) or d.free_award < conf.condition) then
            return true, module_id 
        end
    end
end

return MODULE