-- 等级礼包
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local rechargemod = require "model.rechargemod"
local asset = require "model.asset"
local payment = require "model.payment"
local reward = require "model.reward"
local util = require "util"
local activitytimemod
local currency

local module_name = "level_gift"
local module_id = MODULE_ID_LEVEL_GIFT
local _M = schema.new(module_name, {
    red = false,
    list = {},
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return {errno = STD_ERR.COMMON_ACT_NOT_OPEN} -- 活动未开启
    end
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

local function get_conf(character)
    return activitytimemod.get_conf(character, module_id)
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("recharge.money",function(_, cfid, orderid, moneynum, giftid, id, pos)
        if module_id == id then
            THIS.gift_buy(character, giftid)
        end
    end)

    character.monitor("role_level", function(_, pre, level)
        local d = _M.assert_get(character) or {}
        if not d.red then
            local conf = get_conf(character)
            if conf then
                local gift_conf_list = asset.GiftConfig_proto
                for i = 1, #conf.gift do
                    local id = conf.gift[i]
                    local gift_conf = gift_conf_list[id]
                    if gift_conf and gift_conf.parameter[2] and (gift_conf.parameter[2] > pre and gift_conf.parameter[2] <=level) then
                        d.red = true
                        _M.persist(character)
                        break
                    end
                end
            end
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    activitytimemod = require "model.activitytimemod"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    local ctx = _M.assert_runtime(character)
    ctx.check = true
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
end

function MODULE.open(character, ainfo)
    local red_point = MODULE.check_red_point(character, asset.ActivityConfig_proto[ainfo.sid])
    if red_point then
        local d = _M.assert_get(character)
        d.red = red_point
        _M.persist(character)
    end

end

function MODULE.close(character, ainfo)
end

function MODULE.continue_open(character, ainfo)
end


function THIS.send_nty(character)
    local d = _M.assert_get(character)
    local temp = {}
    for _, v in pairs(d.list) do
        table.insert(temp, v)
    end
    character.send("level_gift_nty", {list = temp})
    return 0
end

function THIS.gift_buy(character, giftid)
    local d = _M.assert_get(character)
    d.list[giftid] = d.list[giftid] or {id = giftid, num = 0}
    d.list[giftid].num = d.list[giftid].num + 1
    _M.persist(character)
    THIS.send_nty(character)
end

function THIS.level_gift_buy(character, args)
    local d = _M.assert_get(character)
    local conf = get_conf(character)

    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local bfind = false
    for i = 1, #conf.gift do
        if conf.gift[i] == id then
            bfind = true
            break
        end
    end

    if not bfind then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if d.list[id] and d.list[id].num >= 1 then
        return STD_ERR.COMMON_HAVE_GIFT -- 已购买
    end
    
    local gift_conf = asset.GiftConfig_proto[id]
    if not gift_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if gift_conf.type ~= module_id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local pay_list = {}
    if gift_conf.spendtype == PAY_TYPE_FREE then
        pay_list = nil
    elseif gift_conf.spendtype == PAY_TYPE_ITEM then 
        table.insert(pay_list, {id = gift_conf.spend, num = gift_conf.num})
    elseif gift_conf.spendtype == PAY_TYPE_GIFT then
        rechargemod.get_orderid(character, gift_conf.ID)
        return 0
    end

    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module=module_name,
            brief="购买等级礼包",
            context= "购买等级礼包"..id,
            mailgen={subject=2, body=""},
            notify={
                flags="level_gift_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end

    local award_list = {}
    for i = 1, #gift_conf.library, 2 do
        local item_id = gift_conf.library[i]
        local item_num = gift_conf.library[i+1]
        if item_id and item_num then
            table.insert(award_list, {id = item_id, num = item_num})
        end
    end

    if next(award_list) then
        -- 发奖励
        local desc = {
            module=module_name,
            brief="购买等级礼包",
            context= "购买等级礼包:"..id,
            mailgen={subject=2, body=""},
            notify={
                flags="level_gift_buy"
            },
            detail=award_list,
        }
        reward(character, desc)
    end
    THIS.gift_buy(character, id)
    return 0
end

function REQUEST.level_gift_data(character, args)
    return func_ret("send_nty", character, args)
end

function REQUEST.level_gift_del_red_point(character, args)
    local d = _M.assert_get(character)
    local ctx = _M.assert_runtime(character)
    d.red = false
    ctx.red = false
    _M.persist(character)
    return {errno = 0}
end


function REQUEST.level_gift_buy(character, args)
    return func_ret("level_gift_buy", character, args)
end

function MODULE.check_red_point(character, conf)
    currency = currency or require "model.currency"
    local d = _M.assert_get(character) or {}
    local conf = conf or get_conf(character)
    local gift_conf_list = asset.GiftConfig_proto

    local dia_num = currency.get_money(character, CURRENCY_ID_DIA)
    for i = 1, #conf.gift do
        local id = conf.gift[i]
        local gift_conf = gift_conf_list[id]
        if gift_conf and not d.list[id] and (gift_conf.parameter[2] and character.level >= gift_conf.parameter[2]) then
            if gift_conf.num <= 0 or (gift_conf.spend == CURRENCY_ID_DIA and gift_conf.num <= dia_num) then
                return true
            end
        end
    end
end

function MODULE.get_red_point(character)
    local state = activitytimemod.get_state(character, module_id)
    if state ~= STATE_OPEN then
        return 
    end

    local d = _M.assert_get(character) or {}
    local ctx = _M.assert_runtime(character)

    if d.red or ctx.red then
        return true, module_id
    end

    if not ctx.check then
        return 
    end

    ctx.check = false

    local red_point = MODULE.check_red_point(character)
    if red_point then
        ctx.red = red_point
        _M.assert_get(character)
        return red_point, module_id
    end

end

return MODULE