-- 千抽活动
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local asset = require "model.asset"
local util = require "util"
local reward = require "model.reward"
local payment = require "model.payment"
local rechargemod = require "model.rechargemod"

local LOTTERY_LOCK = 0 -- 未开启
local LOTTERY_OPEN = 1 -- 开启
local LOTTERY_CLOSE= 2 -- 关闭


local module_name = "lottery"
local module_id = MODULE_ID_LOTTERY
local module_gift = 50048
local _M = schema.new(module_name, {
    state = LOTTERY_LOCK,  -- 0 未开启 1开启 2关闭
    free = 0;   -- 免费奖励
    award = -1; -- 付费奖励 小于0未充值
    time = 0;   -- 开启时间
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("simple_battle_pass", function(character, id)
        local d = _M.assert_get(character) or {}
        if d.state == LOTTERY_LOCK then    
            local data_conf = asset.DataConfig_proto[73]
            if data_conf and data_conf.data5[1] == 1 and data_conf.data5[2] <= id then
                d.state = LOTTERY_OPEN     
                d.time = util.today() + 12 *HOUR_SEC
                _M.persist(character)
            end
        end
    end)

    character.monitor("elite_battle_pass", function(character, id)
        local d = _M.assert_get(character) or {}
        if d.state == LOTTERY_LOCK then    
            local data_conf = asset.DataConfig_proto[73]
            if data_conf and data_conf.data5[1] == 2 and data_conf.data5[2] <= id then
                d.state = LOTTERY_OPEN                    
                d.time = util.today() + 12 *HOUR_SEC
                _M.persist(character)
            end
        end
    end)

    character.monitor("recharge.money",function(_, cfid, orderid, moneynum, giftid, id, pos)
        if module_id == id then
            THIS.gift_buy(character)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character)
    local u = _M.assert_runtime(character)
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)

end

local function get_day(character)
    local d = _M.assert_get(character) or {}
    local day = 0
    if d.state == LOTTERY_OPEN then
        day = math.floor(math.max(0, ((util.today() - util.today(d.time))/DAY_SEC+1)))
    end
    return day
end

function THIS.send_data(character)
    local d = _M.assert_get(character) or {}
    character.send("lottery_nty", {
        state = d.state,
        free = d.free,
        award = d.award,
        day = get_day(character)
    })
end

function THIS.lottery_award(character, args)
    local d = _M.assert_get(character) or {}
    if d.state ~= LOTTERY_OPEN then
        return 11 -- 活动已关闭
    end

    local award = args.free and d.free or d.award
    if award < 0 then
        return 1 -- 未解锁
    end

    local day = get_day(character)
    if award >= day then
        return 2 -- 奖励已领取
    end

    local conf_list = asset.LotteryConfig_proto
    local award_list = {}
    local max_day = 0
    for _, v in pairs(conf_list) do
        max_day = math.max(v.days, max_day)
        if v.days > award and v.days <= day then
            local list = args.free and v.freereward or v.paidreward
            for i = 1, #list, 2 do
                local id = list[i]
                local num = list[i+1]
                table.insert(award_list, {id = id, num = num})
            end
        end
    end

    if next(award_list) then
        -- 发奖励
        local desc = {
            module=module_name,
            brief="领取千抽活动奖励",
            context= string.format("领取千抽活动%s奖励, %d=>%d", args.free and "免费" or "付费", award, day),
            mailgen={subject=2, body=""},
            notify={
                flags="lottery_award"
            },
            detail=award_list,
        }
        reward(character, desc)
    end
    if args.free then
        d.free = math.min(day, max_day)
    else
        d.award = math.min(day, max_day)
    end
    
    if d.free >= max_day and d.award > max_day then
        d.state = LOTTERY_CLOSE
    end
    _M.persist(character)
    return 0, {free = args.free, award = math.min(day, max_day)}
end

function THIS.lottery_data(character, args)
    THIS.send_data(character)
    return 0
end

function THIS.gift_buy(character)
    local d = _M.assert_get(character) or {}
    if d.award < 0 then
        d.award = 0
        _M.persist(character)
        THIS.send_data(character)
    else
        logger.error("重复购买")
    end
end

function THIS.lottery_buy(character, args)
    local d = _M.assert_get(character) or {}
    if d.state ~= LOTTERY_OPEN then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    if d.award >= 0 then
        return 3 -- 已购买
    end

    local gift_conf = asset.GiftConfig_proto[module_gift]
    if not gift_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if gift_conf.type ~= module_id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local pay_list = {}
    if gift_conf.spendtype == PAY_TYPE_FREE then
        pay_list = nil
    elseif gift_conf.spendtype == PAY_TYPE_ITEM then 
        table.insert(pay_list, {id = gift_conf.spend, num = gift_conf.num})
    elseif gift_conf.spendtype == PAY_TYPE_GIFT then
        rechargemod.get_orderid(character, gift_conf.ID)
        return 0
    end

    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module=module_name,
            brief="购买欠抽礼包",
            context= "购买欠抽礼包",
            mailgen={subject=2, body=""},
            notify={
                flags="lottery_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end

    local award_list = {}
    for i = 1, #gift_conf.library, 2 do
        local item_id = gift_conf.library[i]
        local item_num = gift_conf.library[i+1]
        if item_id and item_num then
            table.insert(award_list, {id = item_id, num = item_num})
        end
    end

    THIS.gift_buy(character)
end

function REQUEST.lottery_award(character, args)
    return func_ret("lottery_award", character, args)
end

function REQUEST.lottery_data(character, args)
    return func_ret("lottery_data", character, args)
end

function REQUEST.lottery_buy(character, args)
    return func_ret("lottery_buy", character, args)
end

function MODULE.get_red_point(character)
    local d = _M.assert_get(character) or {}
    if d.state == LOTTERY_OPEN then
        local day = get_day(character)
        local min_day = math.min(d.free , d.award >= 0 and d.award or d.free )
        if min_day < day then
            local conf_list = asset.LotteryConfig_proto
            for _, v in pairs(conf_list) do
                if v.days > min_day and v.days <= day then
                    return true, module_id
                end
            end
        end
    end
    return
end

return MODULE