local schema = require "model.schema"
local logger = require "logger"
local asset = require "model.asset"
local util = require "util"
local payment = require "model.payment"
local reward = require "model.reward"
local common_fun = require "model.common_fun"
local hero

local month_card
local supreme_card
local talent

local MAIN_CITY = 1 -- 主城
local WOOD_CITY = 2 -- 伐木场
local IRON_CITY = 3 -- 铁矿
local COIN_CITY = 4 -- 金矿
local FARM_CITY = 5 -- 农场

local table_insert = table.insert
local math_max = math.max

local module_name = "building"
local _M = schema.new(module_name, {
    skill = 0;
    skill_list = {};
    data = {
        -- [1] = {
        --      id = 1;
        --      level = 0;
        --      time = 时间戳,
        --      auto_explore_option = false;
        --      explore_time = 0;
        --      explore_times = 0;
        --      auto_explore_times = 0;
        -- }
    },
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

-- 探索加速
local function check_explore_speed(character, data)
    local data_conf = asset.DataConfig_proto[59]
    hero = hero or require "model.hero"
    local speed = 0
    for i = 1, #(data.list or {}) do
        local sid = data.list[i]
        if sid ~= "" then
            local hero_data = hero.hero_get(character, sid)
            if hero_data then
                local quality = common_fun.get_hero_quality(hero_data.id)
                for k, v in ipairs(data_conf.data5) do
                    if v == quality then
                        speed = speed + (data_conf.data6[k] or 0)
                    end 
                end
            end
        end
    end
    return speed
end

local function check_explore_change(character, data)
    local speed = check_explore_speed(character, data)
    local data_conf = asset.DataConfig_proto[58]
    local once_time = data_conf.data4*MIN_SEC
    local reduce_time = math.floor((math.min(99, speed)/100)*once_time)
    local explore_time = data.explore_time
    data.explore_time = data.explore_time_start + once_time - reduce_time
    return data.explore_time == explore_time
end

local function chenck_explore_add(character, data, cur)
    if data.id == MAIN_CITY then
        return 0
    end
    local data_conf = asset.DataConfig_proto[58]
    local change = 0
    if data.explore_times >= data_conf.data2 and (not data.auto_explore_option or data.auto_explore_times >= data_conf.data3) then
        data.explore_time = 0
        data.explore_time_start = 0
        change = change + 1
        return change
    else
        local now = cur or os.time()
        if data.explore_time <= 0 or data.explore_time_start <= 0 then
            data.explore_time_start = now
            check_explore_change(character, data)
        end

        if now > data.explore_time then
            if data.auto_explore_option and data.auto_explore_times < data_conf.data3 then
                data.auto_explore_times = data.auto_explore_times + 1
            else
                data.explore_times = data.explore_times + 1
            end
            data.explore_time_start = data.explore_time
            change = change + 1
            check_explore_change(character, data)
            change = change + chenck_explore_add(character, data, now)
        end
        return change
    end
    
end

local function clinet_building_data(data)
    return {
        id = data.id, 
        level = data.level, 
        time = data.time,
        list = data.list or {},
        auto_explore_option = data.auto_explore_option,
        explore_time = data.explore_time,
        explore_times = data.explore_times or 0,
        auto_explore_times = data.auto_explore_times or 0
    }
end

local function cilent_city_data(data)
    local list = {}
    for _, v in pairs(data or {}) do
        local temp = clinet_building_data(v)
        if v.id == MAIN_CITY then
            temp.time = 0
        end
        -- else
        --     temp.time = math_max(0, now - v.time)
        -- end
        table_insert(list, temp)
    end
    return list 
end


function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据
function MODULE.parse(character)
    local d = _M.load(character)
    d.skill_list = d.skill_list or {}
    local data_conf = asset.DataConfig_proto[57]
    if not d.skill or d.skill == 0 then
        d.skill = data_conf.data2
    end

    if #d.skill_list < #data_conf.data5 then
        for _, id in ipairs(data_conf.data5) do
            if id > 0 then
                local skill_type = common_fun.city_skill_type(id)
                if not d.skill_list[skill_type] then
                    d.skill_list[skill_type] = id
                end
            end
        end
    end
    
    d.data = d.data or {}
    local change = false
    local conf_list = asset.BuildingConfig_proto
    local now = os.time()
    local data_conf = asset.DataConfig_proto[58]

    for k, v in pairs(conf_list) do
        if not d.data[v.type] then
            change = true
            d.data[v.type] = {
                id = v.type,
                level = 1,
                time =now
            }
        end
        d.data[v.type].explore_time =  d.data[v.type].explore_time or 0
        d.data[v.type].explore_times =  d.data[v.type].explore_times or data_conf.data2
        d.data[v.type].explore_time_start =  d.data[v.type].explore_time_start or 0
        d.data[v.type].auto_explore_option =  d.data[v.type].auto_explore_option or false
        d.data[v.type].auto_explore_times =  d.data[v.type].auto_explore_times or 0
    end
    if change then
        _M.persist(character)
    end
end



-- TODO: 侦听事件
function MODULE.monitor(character)
    local d = _M.assert_get(character)
    character.monitor("timer.sec", function(_, now)
        local change = 0

        for _, data in pairs(d.data) do
            if data.explore_time > 0 and now > data.explore_time then
                change = change + chenck_explore_add(character, data, now)
            end
        end
        if change > 0 then
            _M.persist(character)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character)
    hero = hero or require "model.hero"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    local ret = 0
    for _, data in pairs(d.data) do
        if data.id ~= MAIN_CITY then
            check_explore_change(character, data)
            ret = chenck_explore_add(character, data, os.time()) + ret
        end
    end
    if ret > 0 then
        _M.persist(character)
    end
    character.send("city_data_send", {list = cilent_city_data(d.data)})
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)

end


function REQUEST.city_data(character, args)
    local d = _M.assert_get(character) or {}
    return {
        errno = 0,
        skillid = d.skill,
        skill_list = util.tablequeue(d.skill_list),
        list = cilent_city_data(d.data),
    }
end

function REQUEST.building_upgrade(character, args)
    local d = _M.assert_get(character) or {}
    local id = args.id
    if not d.data[id] then
        return {errno = STD_ERR.COMMON_PARM_ERR} -- 参数异常
    end

    local fist_level = d.data[MAIN_CITY].level
    if id ~= MAIN_CITY then
        if d.data[id].level >= fist_level then
            return {errno = 1}
        end
    end

    local conf_list = asset.BuildingConfig_proto
    local conf_id = id*10000 + d.data[id].level + 1
    local conf = conf_list[conf_id]
    if not conf or conf.type ~= id then
        return {errno = STD_ERR.COMMON_PARM_ERR} -- 参数异常
    end

    local detail = {}
    for k, v in ipairs(conf.goods) do
        local id = v
        local num = conf.goodsNum[k]
        if not id or not num or num == 0 then
            break 
        end
        table.insert(detail, {id = id, num = num})
    end

    if not next(detail) then
        return {errno = STD_ERR.COMMON_CONF_ERR} --配置异常
    end
    local receipt = {
        module="building",
        brief="建筑升级",
        context="建筑升级:"..conf_id,
        notify={
            flags="building_upgrade",
        },
        detail=detail
    }
    local errno = payment(character, receipt)
    if errno ~= 0 then
        return {errno = errno}
    end
    d.data[id].level = d.data[id].level + 1
    if id ~= MAIN_CITY then
        local data_conf = asset.DataConfig_proto[58]
        if d.data[id].level == data_conf.data1 then
            d.data[id].auto_explore_option = true
            chenck_explore_add(character,  d.data[id])
        end
    end
    _M.persist(character)
    character.dispatch("building_upgrade")
    return {errno = 0, data = clinet_building_data(d.data[id])}
end

-- 农村生产加速
local function get_farm_add(character)
    talent = talent or require "model.talent"
    local add_num = 0
    local talent_list = talent.get_special_attr(character)
    if talent_list then
        add_num = add_num + (talent_list[ATTR_FARM_ADD] or 0)
    end
    return add_num
end

-- 金矿生产加速
local function get_coin_add(character)
    talent = talent or require "model.talent"
    local add_num = 0
    local talent_list = talent.get_special_attr(character)
    if talent_list then
        add_num = add_num + (talent_list[ATTR_COIN_ADD] or 0)
    end
    return add_num
end

function THIS.city_skill_upgrade(character, args)
    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local d = _M.assert_get(character) or {}
    local skill_type = common_fun.city_skill_type(id)
    local skillid = d.skill_list[skill_type]
    if skillid ~= id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local level = d.data[MAIN_CITY].level
    local data_conf = asset.DataConfig_proto[57]
    for i = 1 , #data_conf.data5 do
        if common_fun.city_skill_type(data_conf.data5[i]) == skill_type and level < (data_conf.data6[i] or 0) then
            return STD_ERR.CITY_SKILL_UNLOCK -- 技能未解锁
        end
    end

    local next_id = skillid+1
    local next_conf = asset.CastleSkillConfig_proto[next_id]
    if not next_conf then
        return STD_ERR.CITY_SKILL_LEVEL_MAX -- 无法升级
    end
    local skill_level = common_fun.city_skill_level(id)
    if skill_level >= character.level then
        return STD_ERR.COMMON_NOT_ROLE_LEVEL_LIMIT -- 等级不足
    end

    local pay_list = {}
    for i = 1, #next_conf.upgrade, 2 do
        local item_id = next_conf.upgrade[i]
        local item_num = next_conf.upgrade[i+1]
        if item_id and item_num then
            table.insert(pay_list, {id = item_id, num = item_num})
        end
    end

    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module=module_name,
            brief="城堡技能升级",
            context="城堡技能升级:"..skillid,
            mailgen={subject=2, body=""},
            notify={
                flags="city_skill_upgrade"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end
    d.skill_list[skill_type] = skillid + 1
    if d.skill == skillid then
        d.skill = d.skill_list[skill_type]
    end
    _M.persist(character)
    character.dispatch("city_skill_upgrade", 1)
    return 0, {id = d.skill_list[skill_type]}
end

function THIS.city_skill_battle(character, args)
    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local d = _M.assert_get(character) or {}
    local skill_type = common_fun.city_skill_type(id)
    local skillid = d.skill_list[skill_type]
    if skillid ~= id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local level = d.data[MAIN_CITY].level
    local data_conf = asset.DataConfig_proto[57]
    for i = 1 , #data_conf.data5 do
        if data_conf.data5[i] == id and level < (data_conf.data6[i] or 0) then
            return STD_ERR.CITY_SKILL_UNLOCK -- 技能未解锁
        end
    end

    if d.skill ~= id then        
        d.skill = id
    end
    _M.persist(character)
    return 0, {id = id}
end

function THIS.building_auto_explore_option(character, args)
    local d = _M.assert_get(character)
    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local data = d.data[id]
    if not data then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    if data.auto_explore_option == args.open then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    data.auto_explore_option = args.open
    chenck_explore_add(character, data)
    _M.persist(character)
    return 0, {data = clinet_building_data(data)}
end


function THIS.building_explore_battle(character, args)
    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local d = _M.assert_get(character) or {}
    local data = d.data[id]
    if not data then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end
    
    local list = args.list or {}
    -- 是否已上阵
    if #list > 3 then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end
    local data_conf = asset.DataConfig_proto[59]
    local temp_list = {}
    for i = 1, #list do
        local sid = list[i]
        if sid ~= "" then
            if temp_list[sid] then
                return 6 -- 重复上阵
            end
            -- if data.level < data_conf["data"..i] then
            --     return 4 -- 等级不足
            -- end   
            if not hero.hero_check(character, sid) then
                return 5 -- 未找到英雄
            end
        end
    end

    for _, v in pairs(d.data or {}) do
        if v.id ~= MAIN_CITY and v.id ~= id then
            for i = 1, #(v.list or {}) do
                local sid = v.list[i]
                if sid ~= "" then
                    if temp_list[sid] then
                        return 6 -- 重复上阵
                    end
                end
            end
        end
    end
    data.list = list
    _M.persist(character)
    return 0, {data = clinet_building_data(data)}
end

function THIS.building_auto_explore_award(character, args)
    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end

    local d = _M.assert_get(character) or {}
    local data = d.data[id]
    if not data then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end

    if not data.auto_explore_times or data.auto_explore_times <= 0 then
        return STD_ERR.COMMON_NOT_AWARD  -- 没有奖励
    end

    if id == MAIN_CITY then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end
    
    local conf_id = id*10000 + data.level
    local conf = assert(asset.BuildingConfig_proto[conf_id], "BuildingConfig_proto:"..conf_id)
    local awardid = conf.explore
    if awardid <= 0 then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    local errno, award_list = common_fun.get_award(awardid)
    if errno ~= 0 then
        return errno
    end
    -- award_list = common_fun.award_multiplier(award_list, data.auto_explore_times)
    for i = 1, data.auto_explore_times -1 do
        local temp
        errno, temp = common_fun.get_award(awardid)
        if errno ~= 0 then
            return errno
        end
        award_list = common_fun.merge2list(award_list, temp)
    end


    -- 发奖励
    local desc = {
        module="building",
        brief="自动探索奖励",
        context= "自动探索奖励",
        mailgen={subject=2, body=""},
        notify={
            flags="building_auto_explore_award"
        },
        detail=award_list
    }

    local explore_times = data.auto_explore_times
    data.auto_explore_times = 0
    chenck_explore_add(character, data)
    reward(character, desc)
    _M.persist(character)
    character.dispatch("building_explore", explore_times)
    return 0, {data = clinet_building_data(data)}
end

function REQUEST.building_one_key(character, args)
    local d = _M.assert_get(character)
    local now = os.time()

    month_card = month_card or require "model.activity.month_card"
    supreme_card = supreme_card or require "model.activity.supreme_card"
    talent = talent or require "model.talent"
    local unlock_month_card = month_card.card_unlock(character)
    local unlock_supreme_card = supreme_card.card_unlock(character)
    local talent_list = talent.get_special_attr(character)

    local detail = {}
    local temp_data = common_fun.scopy(d.data)
    for _, v in pairs(temp_data) do
        --主城不产出奖励
        if v.id ~= MAIN_CITY then
            local conf_id = v.id * 10000 + v.level
            local conf = assert(asset.BuildingConfig_proto[conf_id], "BuildingConfig_proto:"..conf_id)
            local once_time = conf.onceTime * 60
            if once_time <= 0 then
                return {errno = STD_ERR.COMMON_CONF_ERR}
            end
            local time = now - v.time
            if time > once_time then
                local max_num = math.floor(asset.DataConfig_proto[2].data1*HOUR_SEC/once_time)
                if v.id == FARM_CITY then
                    local add_limit = 0
                    if unlock_month_card then
                        add_limit = add_limit + asset.DataConfig_proto[47].data1
                    end
                    if unlock_supreme_card then
                        add_limit = add_limit + asset.DataConfig_proto[48].data1
                    end
                    if add_limit > 0 then
                        max_num = max_num + math.floor(add_limit/conf.outNum[1])
                    end
                end
                local add_num = math.floor(time/once_time)
                if add_num >= max_num then
                    v.time = now 
                    add_num = max_num
                else
                    v.time = v.time + add_num*once_time
                end

                local add_redio = 0
                if v.id == WOOD_CITY then
                    if unlock_month_card then
                        add_redio = add_redio + asset.DataConfig_proto[47].data2 * 100
                    end
                    add_redio = (talent_list[ATTR_WOOD_ADD] or 0) + add_redio
                elseif v.id ==IRON_CITY then
                    if unlock_month_card then
                        add_redio = add_redio + asset.DataConfig_proto[47].data3 * 100
                    end
                    add_redio = (talent_list[ATTR_IRON_ADD] or 0) + add_redio
                elseif v.id == WOOD_CITY then
                    add_redio = (talent_list[ATTR_WOOD_ADD] or 0) + add_redio
                elseif v.id == COIN_CITY then
                    add_redio =  (talent_list[ATTR_COIN_ADD] or 0) + add_redio
                end

                for i = 1, #conf.outGoods do
                    local id = conf.outGoods[i]
                    local num = conf.outNum[i]*add_num
                    if add_redio > 0 then
                        num = math.floor(num + num * add_redio / 10000)
                    end
                    if id and num and num > 0 then
                        table.insert(detail, {id = id, num = num})
                    end
                end
            end

        end
    end

    -- 发奖励
    local desc = {
        module="building",
        brief="建筑奖励",
        context= "领取建筑奖励",
        mailgen={subject=2, body=""},
        notify={
            flags="building_one_key"
        },
        detail=detail
    }
    reward(character, desc)
    d.data = temp_data
    _M.persist(character)
    return {errno = 0, list = cilent_city_data(d.data)}
end

function THIS.building_explore(character,args)
    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end

    local d = _M.assert_get(character) or {}
    local data = d.data[id]
    if not data then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end

    if not data.explore_times or data.explore_times <= 0 then
        return STD_ERR.COMMON_NOT_AWARD  -- 没有奖励
    end

    if id == MAIN_CITY then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end
    
    local conf_id = id*10000 + data.level
    local conf = assert(asset.BuildingConfig_proto[conf_id], "BuildingConfig_proto:"..conf_id)
    local awardid = conf.explore
    if awardid <= 0 then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    local errno, award_list = common_fun.get_award(awardid)
    if errno ~= 0 then
        return errno
    end
    -- award_list = common_fun.award_multiplier(award_list, data.explore_times)
    -- 发奖励
    local desc = {
        module="building",
        brief="探索奖励",
        context= "探索奖励",
        mailgen={subject=2, body=""},
        notify={
            flags="building_explore"
        },
        detail=award_list
    }

    -- local explore_times = data.explore_times
    data.explore_times = data.explore_times - 1
    chenck_explore_add(character, data)
    reward(character, desc)
    _M.persist(character)
    character.dispatch("building_explore", 1)
    return 0, {data = clinet_building_data(data)}
end

function REQUEST.city_skill_upgrade(character, args)
    return func_ret("city_skill_upgrade", character, args)
end

function REQUEST.city_skill_battle(character, args)
    return func_ret("city_skill_battle", character, args)
end

function REQUEST.building_auto_explore_option(character, args)
    return func_ret("building_auto_explore_option", character, args)
end

function REQUEST.building_explore_battle(character, args)
    return func_ret("building_explore_battle", character, args)
end

function REQUEST.building_auto_explore_award(character, args)
    return func_ret("building_auto_explore_award", character, args)
end

function REQUEST.building_explore(character, args)
    return func_ret("building_explore", character, args)
end

return MODULE