-- 杂货店 体力,金币购买
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local asset = require "model.asset"

local module_fun = require "model.module_fun"
local payment = require "model.payment"
local reward = require "model.reward"
local common_fun = require "model.common_fun"

local currency

local GROCERY_STM = 1
local GROCERY_COINS = 2

local module_name = "grocery"
local _M = schema.new(module_name, {
    list ={},
    speed_up_time = 0,
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

local function new_data(gtype)
    return {
        type = gtype,
        free_time = 0,
        free_num = 0,
        buy_num = 0,
    }
end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
    d.speed_up_time = d.speed_up_time or 0
    for gtype = GROCERY_STM, GROCERY_COINS do
        if not d.list[gtype] then
            d.list[gtype] = new_data(gtype)
        end
    end
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("daily_refresh", function()
        local d = _M.assert_get(character)
        local change = false
        for _, v in pairs(d.list) do
            if v.free_time > 0 then
                v.free_time = 0
                change = true
            end
            if v.free_num > 0 then
                v.free_num = 0
                change = true
            end

            if v.buy_num > 0 then
                v.buy_num = 0
                change = true
            end
        end
        if change then
            _M.persist(character)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character)
    currency = require "model.currency"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    -- logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)

end

function THIS.grocery_conf(character, gtype)
    local d = _M.assert_get(character) or {}
    local data_conf_list = assert(asset.DataConfig_proto, "DataConfig_proto")
    local conf = {}
    local data = d.list[gtype] or {}
    if gtype == GROCERY_STM then
        conf.free_num = data_conf_list[7].data1
        conf.free_award = {{id = CURRENCY_ID_STM, num = data_conf_list[7].data2}}
        conf.free_cd =  data_conf_list[7].data3

        local buy_num = data.buy_num
        local index = math.min(buy_num+1, #data_conf_list[8].data5)
        conf.buy_pay = {{id = CURRENCY_ID_DIA, num = data_conf_list[8].data5[index]}}
        conf.buy_num = data_conf_list[8].data1
        conf.buy_award = {{id = CURRENCY_ID_STM, num = data_conf_list[8].data2}}
    elseif gtype == GROCERY_COINS then
        conf.free_num = data_conf_list[9].data1
        local redio = 100 + character.level * data_conf_list[9].data4
    
        conf.free_award = {{id = CURRENCY_ID_COINS, num = math.floor(data_conf_list[9].data2*redio/100)}}
        conf.free_cd =  data_conf_list[9].data3

        local buy_num = data.buy_num
        local index = math.min(buy_num+1, #data_conf_list[10].data5)
        conf.buy_pay = {{id = CURRENCY_ID_DIA, num = data_conf_list[10].data5[index]}}
        conf.buy_num = data_conf_list[10].data1
        conf.buy_award = {{id = CURRENCY_ID_COINS, num = math.floor(data_conf_list[10].data2*redio/100)}}
    end
    return conf
end

function THIS.grocery_buy(character, args)
    local gtype = args.type
    local bfree = args.free
    if not gtype then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local conf = THIS.grocery_conf(character, gtype)
    local d = _M.assert_get(character)
    local data = d.list[gtype]
    local pay_list, award_list
    if bfree then
        if data.free_num > conf.free_num then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
        award_list = conf.free_award
    else
        if data.buy_num >= conf.buy_num then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
        pay_list = conf.buy_pay
        award_list = conf.buy_award
    end

    if pay_list and next(pay_list)then
        local receipt = {
            module="grocery",
            brief="杂货购买",
            context=string.format("杂货购买, type:%d", args.type),
            notify={
                flags="grocery_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end

    -- 发奖励
    local desc = {
        module="grocery",
        brief="杂货购买",
        context=string.format("杂货购买, type:%d", args.type),
        mailgen={subject=2, body=""},
        notify={
            flags="grocery_buy"
        },
        detail=award_list
    }
    reward(character, desc)
    if bfree then
        data.free_num = data.free_num + 1
    else
        data.buy_num = data.buy_num + 1
    end
    _M.persist(character)
    if bfree then
        character.dispatch("watch_ad")
    end
    if args.type == GROCERY_STM then
        character.dispatch("buy_stm", bfree)
    end
    return 0, {data = data}
end

function THIS.grocery_data(character, args)
    local d = _M.assert_get(character)
    local gtype = args.type or 0
    local data = d.list[gtype] or {}
    return 0, {data = data}
end

function REQUEST.grocery_buy(character, args)
    return func_ret("grocery_buy", character, args)
end

function REQUEST.grocery_data(character, args)
    return func_ret("grocery_data", character, args)
end


function THIS.box_get_award(character, args)
    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local award_list = nil
    local pay_list = nil
    if args.type == CURRENCY_TYPE_RANDOM_BOX then
        pay_list = currency.get_all_random_box(character)
        if not next(pay_list) then
            return 0
        end
        for _, data in ipairs(pay_list) do
            local Good_conf = assert(asset.GoodsConfig_proto[data.id], "GoodsConfig_proto")
            award_list = award_list or {}
            for i = 1, data.num do
                local errno, temp = common_fun.get_award(Good_conf.award)
                if errno ~= 0 then
                    return errno
                end
                if next(award_list) then
                    award_list = common_fun.merge2list(award_list, temp)
                else
                    award_list = temp
                end
            end
        end
    elseif args.type == CURRENCY_TYPE_SELECT_BOX then
        local id = args.id
        local num = args.num or 1
        if num <= 0 then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
        if not id then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
        pay_list = {{id = id, num = num}}

        local Good_conf = asset.GoodsConfig_proto[id]
        if not Good_conf or Good_conf.type ~= args.type then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end

        if not args.select then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
        local award_conf = asset.AwardConfig_proto[Good_conf.award]
        if not award_conf then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        for i = 1, #award_conf.reward do
            local item_id = award_conf.reward[i]
            local item_num = award_conf.oddsMax[i] * num
            if item_num <= 0 then
                return STD_ERR.COMMON_CONF_ERR -- 配置异常
            end
            if args.select == item_id then
                award_list = award_list or {}
                table.insert(award_list, {id = item_id, num = item_num})
                break
            end
        end
    else
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    if not next(award_list or {}) or not next(pay_list or {}) then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local desc = {
        module="box_get_award",
        brief="领取宝箱奖励",
        context= string.format("领取宝箱奖励"),
        mailgen={subject=2, body=""},
        notify={
            flags="box_get_award"
        },
        detail=award_list
    }

    local receipt = {
        module="box_get_award",
        brief="领取宝箱奖励",
        context= string.format("领取宝箱奖励"),
        mailgen={subject=2, body=""},
        notify={
            flags="box_get_award"
        },
        detail=pay_list,
        must_mail = true,
    }
    local errno = payment(character, receipt)
    if errno ~= 0 then
        return errno
    end

    reward(character, desc)
    return 0, {type = args.type}
end

function THIS.speed_up_buy(character, args)
    local d = _M.assert_get(character) or {}
    local data_conf = assert(asset.DataConfig_proto[11], "DataConfig_proto")
    local add_time = data_conf.data2
    if data_conf.data2<=0 or data_conf.data3 <=0 then
        return STD_ERR.COMMON_CONF_ERR
    end


    local pay_list = {{id = CURRENCY_ID_DIA, num = data_conf.data3}}
    local receipt = {
        module="speed_up_buy",
        brief= "战斗加速购买",
        context= "战斗加速购买",
        mailgen={subject=2, body=""},
        notify={
            flags="speed_up_buy"
        },
        detail = pay_list
    }
    if not args.free then
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end
    local now = os.time()
    d.speed_up_time = d.speed_up_time or 0
    if now > d.speed_up_time then
        d.speed_up_time = now + add_time
    else
        d.speed_up_time = d.speed_up_time + add_time
    end
    _M.persist(character)
    if not args.free then
        character.dispatch("watch_ad")
    end
    return 0, {time = math.max((d.speed_up_time - now), 0)}
end

function REQUEST.box_get_award(character, args)
    return func_ret("box_get_award", character, args)
end

function REQUEST.speed_up_buy(character, args)
    return func_ret("speed_up_buy", character, args)
end

function REQUEST.speed_up_data(character, args)
    local d = _M.assert_get(character) or {}
    local now = os.time()
    if d.speed_up_time == 0 then
        d.speed_up_time = now + asset.DataConfig_proto[11].data4
        _M.persist(character)
    end
    return {errno = 0, time = math.max((d.speed_up_time - now), 0)}
end

return MODULE