local schema = require "model.schema"
local logger = require "logger"
local common_fun = require "model.common_fun"
local module_fun = require "model.module_fun"
local asset = require "model.asset"
local payment = require "model.payment"
local reward = require "model.reward"
local stringify = require "stringify"
-- local keygen   = require "model.keygen"
local embattle 
local equip
local altar

local logger_trace = logger.trace
local table_insert = table.insert
local math_min = math.min

local MAX_NUM = 2000  -- 英雄背包最大数量
local MAX_UPGRADE = 1000 -- 最大升级数量
local module_name = "hero"
local _M = schema.new(module_name, {
    key = 100,
    first = nil,
    num = 0,
    list = {
        --[[
            [sid] = {
                sid = 唯一id,
                id = 模板id,
                lv = 1,
                equip = {穿戴的装备}
            }
        ]]
    },
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local tm_list = {}
tm_list.hero_onekey_upgrade_star = 0
local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    if tm_list[fname] then
        local now = os.time()
        if now - tm_list[fname] == 0 then
            return {errno = STD_ERR.COMMON_CLICK_TOO_FAST} -- 点击太快了
        else
            tm_list[fname] = now
        end
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
    d.key = d.key or 100
    d.num = d.num or 0
    d.list = d.list or {}
    -- if not d.first then
    --     for i = 1, 8 do 
    --         local key = tostring(i)
    --         d.list[key] = {
    --             sid = key,
    --             id = 10301 +100*(i-1),
    --             lv = 1
    --         }
    --     end
    --     for i = 1, 3 do 
    --         local key = tostring(i+10)
    --         d.list[key] = {
    --             sid = key,
    --             id = 10301,
    --             lv = 10
    --         }
    --     end
    --     local key = "21"
    --     d.list[key] = {
    --         sid = key, 
    --         id = 10106,
    --         lv = 10,
    --     }
    --     key = "22"
    --     d.list[key] = {
    --         sid = key, 
    --         id = 10110,
    --         lv = 1,
    --     }
    --     key = "23"
    --     d.list[key] = {
    --         sid = key, 
    --         id = 10114,
    --         lv = 1,
    --     }
    --     d.first = true
    --     d.num = 14
    --     _M.persist(character)
    -- end
end

-- TODO: 侦听事件
function MODULE.monitor(character)

end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character)
    local u = _M.assert_runtime(character)
    embattle = embattle or require "model.embattle"
    equip = equip or require "model.equip"
    altar = altar or require "model.altar"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)

end



function MODULE.hero_surplus(character)
    local d = _M.assert_get(character)
    return math.max((MAX_NUM - d.num), 0)
end

function THIS.key(character)
    local d = _M.assert_get(character)
    d.key = d.key + 1
    return tostring(d.key)
end

local function new_hero(character, id)
    return {
        sid = THIS.key(character),
        id = id,
        lv = 1
    }
end

function MODULE.add_hero(character, id, num, collect, dispose, addition)
    local d = _M.assert_get(character)
    local role_conf_list = assert(asset.RoleConfig_proto, "RoleConfig_proto")
    local conf = role_conf_list[id]
    if not conf and dispose then
        dispose(nil, id, num)
        return 
    end

    local surplus = MODULE.hero_surplus(character)
    if num >= 1 and num <= MAX_NUM then
        local add_num = math.min(surplus, num)
        if add_num > 0 then
            for i = 1, add_num do
                local hero = new_hero(character, id)
                d.list[hero.sid] = hero
                collect(GOODS_HERO, hero)
            end
            d.num = d.num + add_num
            _M.persist(character)
            character.dispatch("hero_add_num", id, add_num)
        end
        addition(GOODS_HERO, id, add_num)
        if add_num < num then
            dispose(GOODS_HERO, id, num - add_num)
        end
        return true, add_num
    else
        if dispose then
            dispose(nil, id, num)
        end
    end
end

function MODULE.hero_check(character, sid)
    local d = _M.assert_get(character)
    return d.list[sid] and true or false 
end

function MODULE.hero_get(character, sid)
    local d = _M.assert_get(character)
    return d.list[sid]
end

function MODULE.hero_get_list(character)
    local d = _M.assert_get(character)
    return d.list
end

function MODULE.hero_save(character)
    _M.persist(character)
end

function THIS.hero_get_data(character, args)
    local d = _M.assert_get(character)
    local ret = {}
    local altar_info = altar.get_altar_info(character)
    local altar_battle = altar.get_altar_battle_info(character)
    for sid, data in pairs(d.list) do
        if altar_battle[sid] then
            local temp = altar.altar_change_hero_info(altar_info, data)
            table.insert(ret, temp)
        else
            table.insert(ret, data)
        end
    end
    return 0, {list = ret}
end

local function get_max_upgrade(character, cur_lv, max_lv)
    local max_lv = max_lv or cur_lv + MAX_UPGRADE
    local level_conf_list = assert(asset.RoleLevelConfig_proto, "RoleLevelConfig_proto")
    local num = 0
    local goods_list = {}
    local flag = true
    for lv = cur_lv+1, max_lv do
        local level_conf = level_conf_list[lv]
        if not level_conf then
            break
        end
        for i = 1, #level_conf.goods do
            local item_id = level_conf.goods[i]
            local item_num = level_conf.goodNum[i]
            if not item_id or not item_num or item_id <=0 or item_num <= 0 then
                break
            end
            if not goods_list[item_id] then
                goods_list[item_id] = module_fun.get_goods_num(character, item_id)
            end
            if goods_list[item_id] < item_num then
                flag = false
                break
            end
            goods_list[item_id] = goods_list[item_id] - item_num
        end
        if not flag then
            break
        end
        num = lv - cur_lv
    end
    return num
end

local function reset_level(lv, list)
    local level_conf_list = assert(asset.RoleLevelConfig_proto, "RoleLevelConfig_proto")
    for lv = 2, lv do
        local level_conf = level_conf_list[lv]
        if not level_conf then
            return STD_ERR.COMMON_SYS_ERR -- 系统异常
        end
        for i = 1, #level_conf.goods do
            local item_id = level_conf.goods[i]
            local item_num = level_conf.goodNum[i]
            if item_id and item_num and item_num > 0 then
                table_insert(list, {id = item_id, num = item_num})
            else
                break
            end
        end
    end
    return 0
end


function THIS.hero_upgrade(character, args)
    local d = _M.assert_get(character)
    local sid = args.sid 
    local num = args.num or 0
    if not sid then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end

    if num < 0 or num > MAX_UPGRADE then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local altar_battle = altar.get_altar_battle_info(character)
    if altar_battle[sid] then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end

    local data = d.list[sid]
    local level = data.lv
    if not data then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local hero_conf_list = assert(asset.RoleConfig_proto, "RoleConfig_proto")
    local hero_conf = hero_conf_list[data.id]
    if not hero_conf then
        logger_trace("RoleConfig_proto:%d", data.id)
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    -- if hero_conf.type == HERO_TYPE_CITY or hero_conf.type == HERO_TYPE_CARD then
    --     return 4 -- 异常类型
    -- end

    local quality_conf_list = assert(asset.RoleQualityConfig_proto, "RoleQualityConfig_proto")
    local quality_conf = quality_conf_list[hero_conf.quality]
    if not quality_conf then
        logger_trace("RoleQuatityConfig_proto:%d", hero_conf.quality)
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    
    if num == 0 then
        num = get_max_upgrade(character, level, math_min(quality_conf.maxLv, character.level))
        if num <= 0 then
            return STD_ERR.HERO_MAX_LEVEL -- 超过最大等级
        end
    end

    if level+num > quality_conf.maxLv or level +num > character.level then
        return STD_ERR.HERO_MAX_LEVEL -- 超过最大等级
    end


    local level_conf_list = assert(asset.RoleLevelConfig_proto, "RoleLevelConfig_proto")
    local pay_list = {}
    for i = 1, num do 
        local conf = level_conf_list[level+i]
        if not conf then
            return STD_ERR.COMMON_CONF_ERR-- 配置异常
        end
        for j = 1, #conf.goods do
            local item_id = conf.goods[j]
            local item_num = conf.goodNum[j]
            if item_id and item_num then
                table_insert(pay_list, {id = item_id, num = item_num})
            end
        end
    end
    local receipt = {
        module="hero",
        brief="角色升级",
        context=string.format("角色id%d, 从%d升级到%d", data.id, level, level+num),
        notify={
            flags="hero_upgrade"
        },
        detail=pay_list
    }
    local errno = payment(character, receipt)
    if errno ~=0 then
        return errno
    end

    local cur_lv = data.lv+num
    local pre_lv = data.lv
    data.lv = data.lv+num
    _M.persist(character)
    character.dispatch("hero_upgrade", data.id, pre_lv, cur_lv)
    character.dispatch("hero_upgrade_info", data)
    return 0, {data = data}
end


function THIS.hero_upgrade_star(character, args)
    local d = _M.assert_get(character)
    local sid = args.sid
    local same_list = args.same or {}
    local other_list = args.other or {}
    local item_list = args.currency or {}
    local buse_item = next(item_list) and item_list.num and item_list.num > 0 and true or false
    if buse_item then
        if not item_list.id then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
    end

    local altar_battle = altar.get_altar_battle_info(character)
    if altar_battle[sid] then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end

    if not sid then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local data = d.list[sid]
    if not data then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    -- 先检查是否重复
    local battle_list = embattle.get_embattle_hero_list(character)
    local equip_list = {}
    local temp_list = {[sid] = 1}
    for _, v in ipairs(same_list) do
        if temp_list[v] then
            return STD_ERR.HERO_SAME_SID -- 重复的角色id
        end
        if battle_list[v] then
            return STD_ERR.HERO_IN_BATTLE -- 阵上的不能被消耗
        end

        if altar_battle[v] then
            return STD_ERR.HERO_IN_BATTLE -- 阵上的不能被消耗
        end

        if not d.list[v] then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
        temp_list[v] = 1
        for _, esid in ipairs(d.list[v].equip or {}) do
            if esid and esid ~= "" then
                table_insert(equip_list, esid)
            end
        end
    end

    for _, v in ipairs(other_list) do
        if temp_list[v] then
            return STD_ERR.HERO_SAME_SID -- 重复的角色id
        end
        if battle_list[v] then
            return STD_ERR.HERO_IN_BATTLE -- 阵上的不能被消耗
        end

        if altar_battle[v] then
            return STD_ERR.HERO_IN_BATTLE -- 阵上的不能被消耗
        end

        if not d.list[v] then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
        temp_list[v] = 1
        for _, esid in ipairs(d.list[v].equip or {}) do
            if esid and esid ~= "" then
                table_insert(equip_list, esid)
            end
        end
    end

    local ok = equip.equip_remove_check(character, equip_list)
    if not ok then
        return STD_ERR.EQUIP_FULL
    end

    local role_conf_list = assert(asset.RoleConfig_proto, "RoleConfig_proto")
    local role_conf = role_conf_list[data.id]
    if not role_conf then
        logger_trace("RoleConfig_proto:%d", data.id)
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    
    local cur_quality = role_conf.quality
    if not cur_quality then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    local max_quality = asset.DataConfig_proto[5].data1
    if cur_quality >= max_quality then
        return STD_ERR.HERO_START_MAX -- 达到最大突破等级
    end


    -- local conf_id = -1
    -- local min_id = math.floor(data.id/100)*100+1
    -- local max_id = min_id + 98
    -- for i = min_id, max_id do
    --     local conf = role_conf_list[i]
    --     if conf then
    --         if conf.ID >= min_id and conf.ID <= max_id and conf.quality == cur_quality+1 then
    --             conf_id = conf.ID
    --             break
    --         end
    --     end
    -- end
    -- if conf_id <= 0 then
    --     return STD_ERR.COMMON_CONF_ERR -- 配置异常
    -- end

    local min_id = math.floor(data.id/100)*100+1
    local max_id = min_id + 98
    local conf_id = data.id + 1
    if not role_conf_list[conf_id] or role_conf_list[conf_id].quality ~= cur_quality+1 then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    
    local quality_conf_list = assert(asset.RoleQualityConfig_proto, "RoleQualityConfig_proto")
    local quality_conf = quality_conf_list[cur_quality+1]
    if not quality_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    -- 检查消耗数量
    if quality_conf.sameQualityNum ~= #same_list or quality_conf.anyQualityNum ~= #other_list + (buse_item and item_list.num or 0) then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    --检查同名英雄消耗
    for i = 1, #same_list do
        local id = d.list[same_list[i]].id
        if id < min_id or id > max_id then
            return STD_ERR.HERO_SAME_PAY_ERR -- 不符合消耗条件
        end
        local conf = role_conf_list[id]
        if conf.quality ~= quality_conf.sameQuality then
            return STD_ERR.HERO_SAME_PAY_ERR -- 不符合消耗条件
        end
    end

    --检查任意英雄消耗
    for i = 1, #other_list do
        local id = d.list[other_list[i]].id
        local conf = role_conf_list[id]
        if conf.quality ~= quality_conf.anyQuality then
            return STD_ERR.HERO_OTHER_PAY_ERR -- 不符合消耗条件
        end
    end

    -- 检查替代品消耗
    if buse_item then
        if quality_conf.resetGoods ~= item_list.id then
            return STD_ERR.HERO_OTHER_PAY_ERR -- 不符合消耗条件
        end
        local receipt = {
            module="hero",
            brief="角色升星",
            context=string.format("角色id%d, 升星为%d", data.id, conf_id),
            notify={
                flags="hero_upgrade_star"
            },
            detail={item_list}
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end

    local del_list = {}
    local award_list = {}
    -- 把突破的英雄排除
    temp_list[sid] = nil
    for k in pairs(temp_list) do
        if d.list[k].lv > 1 then
            reset_level(d.list[k].lv, award_list)
        end
        d.list[k] = nil
        table_insert(del_list, k)
    end 

    if next(award_list) then
        local desc = {
            module="hero",
            brief="英雄升星",
            context= "升星返回",
            mailgen={subject=2, body=""},
            notify={
                flags="hero_upgrade_star"
            },
            detail=award_list
        }
        reward(character, desc)
    end
    equip.equip_remove(character, equip_list)
    d.list[sid].id = conf_id
    d.num = d.num - #del_list
    _M.persist(character)
    character.dispatch("hero_del", del_list)
    character.dispatch("hero_upstart", conf_id, cur_quality, cur_quality+1)
    character.dispatch("hero_upstart_info", d.list[sid])
    return 0, {data = d.list[sid], same = args.same, other = args.other, currency = args.currency}
end

function THIS.hero_onekey_upgrade_star(character, args)
    local d = _M.assert_get(character)
    local temp_list = {}
    local battle_list = embattle.get_embattle_hero_list(character)
    local altar_battle = altar.get_altar_battle_info(character)
    for sid, data in pairs(d.list) do
        -- 紫色 未上阵 未升级的英雄一键突破
        local quality = data.id%100
        if not battle_list[sid] and not altar_battle[sid] and data.lv == 1 and quality < STAR_COLOR_PURPLE and not data.equip then
            temp_list[quality] = temp_list[quality] or {}
            temp_list[quality][data.id] = temp_list[quality][data.id] or {}
            table_insert(temp_list[quality][data.id], sid)
        end
    end

    local del_list = {}
    local change_list = {}
    local max_quality = 1
    for quality = 1, STAR_COLOR_PURPLE do
        local quality_conf_list = assert(asset.RoleQualityConfig_proto, "RoleQualityConfig_proto")
        if not quality_conf_list[quality] or quality_conf_list[quality].anyQualityNum > 0 then
            break
        end
        local temp = temp_list[quality]
        if temp then
            for id, list in pairs(temp) do
                local quality_conf = quality_conf_list[quality+1]
                local role_conf = assert(asset.RoleConfig_proto, "RoleConfig_proto")[id+1]
                local num = #list
                local step = quality_conf.sameQualityNum+1
                assert(step > 0)
                if role_conf and role_conf.quality == quality + 1 and step <= num and 
                    quality_conf.sameQuality == quality
                then
                    for i = 1, num, step do
                        if list[i+step-1] then
                            d.list[list[i]].id = id+1
                            table_insert(change_list, d.list[list[i]])
                            for j = i+1, i+step-1 do
                                d.list[list[j]] = nil
                                table_insert(del_list, list[j])
                            end
                            max_quality = quality
                            if #change_list >= 100 then
                                goto hero_onekey_upgrade_star_break
                            end
                        end
                    end
                end

            end    
        end
    end
    ::hero_onekey_upgrade_star_break::
    if not next(change_list) then
        return STD_ERR.HERO_NOBODY_CAN_UP_STAR -- 没有
    end
    d.num = d.num - #del_list
    _M.persist(character)
    character.dispatch("hero_del", del_list)
    character.dispatch("hero_upstart_times", #(change_list), max_quality + 1)

    return 0, {del_list = del_list, change_list = change_list}
end

-- 
function THIS.hero_reset(character, args)
    local sid = args.sid 
    if not sid then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local d = _M.assert_get(character)
    local data = d.list[sid]
    if not data then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local temp_data = common_fun.scopy(data)
    local award_list = {}
    local desc = {
        module="hero",
        brief="重置英雄",
        mailgen={subject=2, body=""},
        notify={
            flags="hero_reset"
        },
    }
    if args.level then
        if data.lv <= 1 then
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
            -- 发奖励
        desc.context= string.format("重置英雄等级, sid:%s, lv:%d", sid, data.lv)
    else
        local role_conf_list = assert(asset.RoleConfig_proto, "RoleConfig_proto")
        local role_conf = role_conf_list[data.id]
        if not role_conf then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end

        local quality = common_fun.get_hero_quality(data.id)
        if quality ~= role_conf.quality then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        
        local min_quality = -1
        if quality > STAR_COLOR_RED then
            min_quality = STAR_COLOR_RED
        elseif quality > STAR_COLOR_ORANGE then
            min_quality = STAR_COLOR_ORANGE
        elseif quality > STAR_COLOR_PURPLE then
            min_quality = STAR_COLOR_PURPLE
        else
            return STD_ERR.COMMON_PARM_ERR -- 参数异常
        end
        local quality_conf_list = assert(asset.RoleQualityConfig_proto, "RoleQualityConfig_proto")
        for i = min_quality+1, quality do
            local quality_conf = quality_conf_list[i]
            if not quality_conf or quality_conf.sameQualityNum>0 then
                return STD_ERR.COMMON_CONF_ERR -- 配置异常
            end
            table_insert(award_list, {id = quality_conf.resetGoods, num = quality_conf.anyQualityNum})
        end
        local new_id = math.floor(data.id /100)*100+min_quality
        role_conf = role_conf_list[new_id]
        if not role_conf or role_conf.quality ~= min_quality then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        desc.context = string.format("重置英雄突破等级, sid:%s, id:%d==>%d", sid, data.id, new_id)
        temp_data.id = new_id
    end

    local equip_list = {}
    for _, esid in ipairs(data.equip or {}) do
        if esid and esid ~= nil  then
            table_insert(equip_list, esid)
        end
    end
    local bremove = equip.equip_remove_check(character, equip_list)

    if data.lv > 1 then
        local level_conf_list = assert(asset.RoleLevelConfig_proto, "RoleLevelConfig_proto")
        for lv = 2, data.lv do
            local level_conf = level_conf_list[lv]
            if not level_conf then
                logger_trace("RoleLevelConfig_proto"..lv)
                return STD_ERR.COMMON_SYS_ERR -- 系统异常
            end
            for i = 1, #level_conf.goods do
                local item_id = level_conf.goods[i]
                local item_num = level_conf.goodNum[i]
                if item_id and item_num and item_num > 0 then
                    table_insert(award_list, {id = item_id, num = item_num})
                else
                    break
                end
            end
        end
        temp_data.lv = 1
    end
    desc.detail = award_list
    reward(character, desc)
    d.list[sid] = temp_data
    if bremove then
        equip.equip_remove(character, equip_list)
        d.list[sid].equip = nil
    end
    _M.persist(character)
    character.dispatch("hero_reset_info", d.list[sid])
    return 0, {data = temp_data}
end

function THIS.hero_wear_equip(character, args)
    local sid = args.sid
    local list = args.list or {}
    local d = _M.assert_get(character)
    if not sid or not d.list[sid] then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local data = d.list[sid]

    -- -- 清除冗余字段
    -- for i = #list , 1, -1 do
    --     if list[i] == "" then
    --         list[i] = nil
    --     else
    --         break
    --     end
    -- end
    if #list ~= 6 and #list ~= 0 then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local errno = equip.equip_wear(character, data, data.equip, list)
    if errno ~= 0 then
        return errno
    end
    local empty = true
    -- 清除冗余字段
    for i = #list , 1, -1 do
        if list[i] ~= "" then
            empty = false
            break
        end
    end
    if empty then
        list = nil
    end
    data.equip = list
    _M.persist(character)
    return 0, {list = args.list, data = data}
end

-- 英雄背包数据
function REQUEST.hero_get_data(character, args)
    return func_ret("hero_get_data", character, args)
end

-- 英雄升级
function REQUEST.hero_upgrade(character, args)
    return func_ret("hero_upgrade", character, args)
end

-- 英雄升星
function REQUEST.hero_upgrade_star(character, args)
    return func_ret("hero_upgrade_star", character, args)
end

-- 一键升星
function REQUEST.hero_onekey_upgrade_star(character, args)
    return func_ret("hero_onekey_upgrade_star", character, args)
end

-- 重置
function REQUEST.hero_reset(character, args)
    return func_ret("hero_reset", character, args)
end

-- 装备穿戴
function REQUEST.hero_wear_equip(character, args)
    return func_ret("hero_wear_equip", character, args)
end

-- 英雄的特殊属性
function MODULE.get_special_attr(character)
    local d = _M.assert_get(character)
    local battle_list = embattle.get_embattle_hero_list(character)

    local list = {}
    for sid in pairs(battle_list or {}) do
        local data = d.list[sid]
        if not data then
            goto CONTINUE
        end

        for _, esid in ipairs(data.equip or {}) do
            if not esid or esid == "" then
                goto CONTINUE2
            end
            local equip_info = equip.get_equip_data(character, esid)
            if not equip_info then
                goto CONTINUE2
            end

            local equip_conf = asset.ArmorConfig_proto[equip_info.id]
            if equip_conf and equip_conf.entry and equip_conf.entry > 0 then
                local conf = asset.EntryConfig_proto[equip_conf.entry]
                if conf and conf.type and conf.type >= 100 then
                    if conf.rateArr[1] and conf.rateArr[1] > 0 then
                        list[conf.type] = (list[conf.type] or 0) + conf.rateArr[1]
                    end
                end
            end
            ::CONTINUE2::
        end
        ::CONTINUE::
    end
    logger.test("equip get_special_attr"..stringify(list or {}))
    return list
end

return MODULE