local schema = require "model.schema"
local logger = require "logger"
local asset = require "model.asset"
local stringify = require "stringify"
local util = require "util"
local reward = require "model.reward"
local payment = require "model.payment"
local rechargemod = require "model.rechargemod"
local common_fun = require "model.common_fun"

local GIFT_SHOP     = 1 -- 礼包商店 
local DAILY_SHOP    = 2 -- 每日商店
local DIA_SHOP      = 3 -- 钻石商店
local SCORE_SHOP    = 4 -- 积分商店

-- 模块id对应的商店类型
local modleid_list = {
    [2006] = GIFT_SHOP, 
    [2007] = GIFT_SHOP, 
    [2008] = GIFT_SHOP, 
    [MODULE_ID_DIA_SHOP] = DIA_SHOP, 
}

local module_name = "shop"
local module_id = MODULE_ID_SHOP
local _M = schema.new(module_name, {
    list = {
    --[[
        [商店类型] = 
        {
            [id] = {
                id = 物品id,
                num = 已购买数量,
            }
        }
    ]]

    }
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_basic_init(list, conf_list)
    local change = false
    for id in pairs(list) do
        if not conf_list[id] then
            list[id] = nil
            change = true
        end
    end
    return change
end

local function get_goods_conf(shop_type, id)
    local list = {}
    if shop_type == GIFT_SHOP or shop_type == DIA_SHOP then
        local conf = asset.GiftConfig_proto[id]
        if not conf then
            return list
        end
        list.pay_type = conf.spendtype
        list.pay_list = {conf.spend, conf.num}
        list.award_list = conf.library
        list.awardid = conf.award
        list.limit = conf.daylimit > 0 and conf.daylimit or conf.weeklimit
    elseif shop_type == DAILY_SHOP or shop_type == SCORE_SHOP then
        local conf
        if shop_type == DAILY_SHOP then
            conf = asset.ItemshopConfig_proto[id]
        elseif shop_type == SCORE_SHOP then
            conf = asset.IntegralshopConfig_proto[id]
        end
        if not conf then
            return list
        end
        list.pay_type = PAY_TYPE_ITEM
        if not conf.price or conf.price[1] == 0 then
            list.pay_type = PAY_TYPE_FREE
        else
            list.pay_list = conf.price
        end
        list.award_list = conf.item
        list.limit = conf.num
    end
    return list
end

local function daily_shop_reset(character, list, conf_list)
    local num = asset.DataConfig_proto[19].data2
    local temp_conf_list = {}
    for _, conf in pairs(conf_list) do
        if conf then
            if character.level > conf.need then
                if conf.must == 1 then
                    list[conf.ID] = {
                        id = conf.ID,
                        num = 0,
                    }
                    num = num - 1
                else
                    table.insert(temp_conf_list, conf)
                end
            end
        end
    end

    while(num > 0) do
        local size = #temp_conf_list
        if size <= 0 then
            break
        end
        local index = 1
        if size > 1 then
            index = util.rand(1, size)        
        end
        local conf = temp_conf_list[index]
        table.remove(temp_conf_list, index)
        list[conf.ID] = {
            id = conf.ID,
            num = 0,
        }
        num = num - 1
    end
end

local func_init_shop = {}
func_init_shop[GIFT_SHOP] = function(character)
    local d = _M.assert_get(character)
    local shop_type = GIFT_SHOP
    local conf_list = asset.GiftConfig_proto
    d.list[shop_type] = d.list[shop_type] or {}
    local list = d.list[shop_type]
    local change = func_basic_init(list, conf_list)
    return change
end

func_init_shop[DAILY_SHOP] = function(character)
    local d = _M.assert_get(character)
    local shop_type = DAILY_SHOP
    local conf_list = asset.ItemshopConfig_proto
    d.list[shop_type] = d.list[shop_type] or {}
    local list = d.list[shop_type]
    local change = func_basic_init(list, conf_list)
    if not next(list) then
        daily_shop_reset(character, list, conf_list)
        change = true
    end
    return change
end

func_init_shop[DIA_SHOP] = function(character)
    local d = _M.assert_get(character)
    local shop_type = DIA_SHOP
    local conf_list = asset.GiftConfig_proto
    d.list[shop_type] = d.list[shop_type] or {}
    local list = d.list[shop_type]
    local change = func_basic_init(list, conf_list)
    return change
end

func_init_shop[SCORE_SHOP] = function(character)
    logger.trace("SCORE_SHOP")
    local d = _M.assert_get(character)
    local shop_type = SCORE_SHOP
    local conf_list = asset.IntegralshopConfig_proto
    d.list[shop_type] = d.list[shop_type] or {}
    local list = d.list[shop_type]
    local change = func_basic_init(list, conf_list)
    return change
end

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("recharge.money",function(_, cfid, orderid, moneynum, giftid, modleid)
        local shop_type = modleid_list[modleid]
        if shop_type then
            local d = _M.assert_get(character) or {}
            d.list[shop_type][giftid] = d.list[shop_type][giftid] or {id = giftid, num = 0} 
            local data = d.list[shop_type][giftid]
            data.num = data.num + 1
            _M.persist(character)
            if shop_type == DIA_SHOP then
                local data = d.list[shop_type][giftid]
                local conf = asset.GiftConfig_proto[giftid]
                local award_list
                if conf.award and conf.award > 0 then
                    local errno = 0
                    errno, award_list = common_fun.get_award(conf.award, "NewawardConfig_proto")
                    if errno ~= 0 then
                        logger.warn("计费点奖励异常, errno:%d, giftid:%d, awardid:%d", errno, giftid, conf.award)
                        return 
                    end
                end
                award_list = award_list or {}
                for i = 1, #conf.library, 2 do
                    local item_id = conf.library[i]
                    local item_num = conf.library[i+1]
                    if item_id and item_num then
                        table.insert(award_list, {id = item_id, num = item_num})
                    end
                end
                -- 首充双倍
                if data.num == 1 then
                    award_list = common_fun.award_multiplier(award_list, 2)
                end
                -- 发奖励
                local desc = {
                    module="shop_buy",
                    brief="购买商店物品",
                    context= "shop_buy:"..giftid,
                    mailgen={subject=2, body=""},
                    notify={
                        flags="recharge_money"
                    },
                    detail=award_list
                }
                reward(character, desc)
            end
            character.send("shop_notify", {type = shop_type, id = giftid, num = 1})
        end
    end)

    character.monitor("daily_refresh", function()
        local d = _M.assert_get(character)
        local data = d.list[GIFT_SHOP]
        local conf_list = asset.GiftConfig_proto
        for id, v in pairs(data or {}) do
            local conf = conf_list[id]
            if conf and conf.daylimit > 0 then
                v.num = 0
            end
        end

        data = d.list[DIA_SHOP]
        for id, v in pairs(data or {}) do
            local conf = conf_list[id]
            if conf and conf.daylimit > 0 then
                v.num = 0
            end
        end

        data = d.list[SCORE_SHOP]
        conf_list = asset.IntegralshopConfig_proto
        for id, v in pairs(data or {}) do
            local conf = conf_list[id]
            if conf and conf.limit == LIMIT_TYPE_DAILY then
                v.num = 0
            end
        end

        d.list[DAILY_SHOP] = {}
        data = d.list[DAILY_SHOP]
        conf_list = asset.ItemshopConfig_proto
        daily_shop_reset(character, data, conf_list)
        _M.persist(character)
    end)
    
    character.monitor("weekly_refresh", function()
        local d = _M.assert_get(character) or {}
        local data = d.list[GIFT_SHOP]
        local conf_list = asset.GiftConfig_proto
        for id, v in pairs(data or {}) do
            local conf = conf_list[id]
            if conf and conf.weeklimit > 0 then
                v.num = 0
            end
        end

        data = d.list[DIA_SHOP]
        for id, v in pairs(data or {}) do
            local conf = conf_list[id]
            if conf and conf.weeklimit > 0 then
                v.num = 0
            end
        end

        data = d.list[SCORE_SHOP]
        conf_list = asset.IntegralshopConfig_proto
        for id, v in pairs(data or {}) do
            local conf = conf_list[id]
            if conf and conf.limit == LIMIT_TYPE_WEEKLY then
                v.num = 0
            end
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    local change = false
    for shop_type = GIFT_SHOP, SCORE_SHOP do
        local f = func_init_shop[shop_type]
        if f then
            change = f(character) or change
        end
    end
    if change then
        _M.persist(character)
    end
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)

end

function THIS.shop_data(character, args)
    local d = _M.assert_get(character) or {}
    local ret = {}
    for shop_type, list in pairs(d.list or {}) do
        local index = #ret + 1
        ret[index] = {
            type = shop_type,
            list = {}
        }
        for _, v in pairs(list) do
            table.insert(ret[index].list, v)
        end
    end
    return 0, {list = ret}
end

function THIS.shop_buy(character, args)
    local shop_type = args.type
    local id = args.id
    local num = args.num
    local conf = get_goods_conf(shop_type, id)
    if not conf or not next(conf) then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local d = _M.assert_get(character)
    if not d.list[shop_type] then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    
    local data = d.list[shop_type][id] or {id = id, num = 0} 
    if conf.pay_type == PAY_TYPE_GIFT then
        num = 1
    end

    if conf.limit > 0 and data.num + num > conf.limit then
        return 2 -- 数量不足
    end

    local pay_list = {}
    local award_list= {}
    if conf.pay_type == PAY_TYPE_FREE then
        pay_list = nil
    elseif conf.pay_type == PAY_TYPE_ITEM then
        for i = 1, #conf.pay_list, 2 do
            local item_id = conf.pay_list[i]
            local item_num = conf.pay_list[i+1]
            if item_id and item_num then
                table.insert(pay_list, {id = item_id, num = item_num})
            end
        end
    elseif conf.pay_type == PAY_TYPE_GIFT then
        rechargemod.get_orderid(character, id)
        return 0
    end

    -- 限制下, 免费礼包不可无限购买
    if conf.limit <= 0 and conf.pay_list == PAY_TYPE_FREE then
        return STD_ERR.COMMON_CONF_ERR
    end

    for i = 1, #conf.award_list, 2 do
        local item_id = conf.award_list[i]
        local item_num = conf.award_list[i+1]
        if item_id and item_num then
            table.insert(award_list, {id = item_id, num = item_num})
        end
    end

    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module="shop_buy",
            brief="购买商店物品",
            context= string.format("购买商店物品, type: %d, id:%d, num", shop_type, id, num),
            mailgen={subject=2, body=""},
            notify={
                flags="shop_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end

    if award_list and next(award_list) then
        local desc = {
            module="shop_buy",
            brief="购买商店物品",
            context= string.format("购买商店物品, type: %d, id:%d, num:%d", shop_type, id, num),
            mailgen={subject=2, body=""},
            notify={
                flags="shop_buy"
            },
            detail=award_list
        }
        reward(character, desc)
    end
    data.num = data.num + num
    d.list[shop_type][id] = data
    _M.persist(character)
    character.send("shop_notify", {type = args.type, id = id, num = num})
    character.dispatch("shop_buy", id, num)
    if conf.pay_type == PAY_TYPE_FREE then
        character.dispatch("watch_ad")
    end
    return 0
end

function REQUEST.shop_data(character, args)
    return func_ret("shop_data", character, args)
end

function REQUEST.shop_buy(character, args)
    return func_ret("shop_buy", character, args)
end

function MODULE.get_red_point(character)
    local d = _M.assert_get(character) or {}

    -- 礼包商店检查
    for _, gift_conf in pairs(asset.GiftConfig_proto) do
        local shop_type = modleid_list[gift_conf.type]
        if shop_type then
            local shop_data = d.list[shop_type]
            local conf = get_goods_conf(shop_type, gift_conf.ID)
            if shop_data and conf and conf.pay_type == PAY_TYPE_FREE then
                if not shop_data[gift_conf.ID] or shop_data[gift_conf.ID].num < conf.limit then
                    return true, module_id
                end
            end
        end
    end
    -- 积分商店检查
    local shop_type = SCORE_SHOP
    for _, gift_conf in pairs(asset.IntegralshopConfig_proto) do
        local shop_data = d.list[shop_type]
        local conf = get_goods_conf(shop_type, gift_conf.ID)
        if shop_data and conf and conf.pay_type == PAY_TYPE_FREE then
            if not shop_data[gift_conf.ID] or shop_data[gift_conf.ID].num < conf.limit then
                return true, module_id
            end
        end
    end

    -- 每日商店检查
    for _, data in pairs(d.list[DAILY_SHOP] or {}) do
        local conf = get_goods_conf(DAILY_SHOP, data.id)
        if conf and conf.pay_type == PAY_TYPE_FREE and data.num < conf.limit then
                return true, module_id
        end
    end

end

return MODULE