local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local common_fun = require "model.common_fun"
local module_fun = require "model.module_fun"
local payment = require "model.payment"
local asset = require "model.asset"
local debris

local logger_trace = logger.trace
local math_floor = math.floor
local math_min = math.min

local MAX_UPGRADE = 1000
local MAX_NUM = 200000000
local module_name = "skill_card"
local _M = schema.new(module_name, {
    list = {}
    --[[
        list = {
            [sid] = {
                sid = sid,
                id = id,
                lv = lv,
            }
        }
    ]]
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
    d.list = d.list or {}
end

-- TODO: 侦听事件
function MODULE.monitor(character)

end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character)
    local u = _M.assert_runtime(character)
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
end

local function new_skill_card(key, id)
    return {
        sid = key,
        id = id, 
        lv = 1,
    }
end

function MODULE.add_skill_card(character, id, num, collect, dispose, addition)
    local d = _M.assert_get(character)
    local conf_list = assert(asset.CardSkillConfig_proto, "CardSkillConfig_proto")
    if not conf_list[id] or num < 1 or num > MAX_NUM then
        dispose(nil, id, num)
    end
    local key_num = (math.floor(id/100))
    local key = tostring(key_num)
    if not d.list[key] then
        d.list[key] = new_skill_card(key, id)
        collect(GOODS_CARD, d.list[key])
        addition(GOODS_CARD, id, 1)
        num = num -1
        _M.persist(character)
        character.dispatch("skill_card_add_num", 1)
    end
    -- 计算转化
    if num > 0 then
        debris = debris or require "model.debris"
        local debris_num = 0
        for i = key_num*100+1, id do
            local conf = asset.CardSkillConfig_proto[i]
            if not conf then
                dispose(nil, id, num)
                break
            else
                debris_num = debris_num + conf.consume
            end
        end
        debris.add_debris(character, conf_list[id].debris, debris_num , collect, dispose, addition)
    end
    return 0
end

-- 检查是否拥有技能
function MODULE.skill_card_check(character, id)
    local d = _M.assert_get(character)
    return d.list[id] and true or false
end

-- 检查该类型技能是否激活
function MODULE.skill_card_check_skill(character, id)
    local d = _M.assert_get(character)
    local key = tostring(math_floor(id/100))
    return d.list[key] and true or false
end

function MODULE.skill_card_get(character, sid)
    local d = _M.assert_get(character)
    return d.list[sid]
end


function THIS.skill_card_get_data(character, args)
    local d = _M.assert_get(character)
    return 0, {list = common_fun.tqueue(d.list)} 
end

local function get_max_upgrade(character, cur_lv, max_lv)
    local max_lv = max_lv or cur_lv + MAX_UPGRADE
    local level_conf_list = assert(asset.CardLevelConfig_proto, "CardLevelConfig_proto")
    local num = 0
    local goods_list = {}
    local flag = true
    for lv = cur_lv+1, max_lv do
        local level_conf = level_conf_list[lv]
        if not level_conf then
            break
        end
        for i = 1, #level_conf.goods do
            local item_id = level_conf.goods[i]
            local item_num = level_conf.goodNum[i]
            if not item_id or not item_num or item_id <=0 or item_num <= 0 then
                break
            end
            if not goods_list[item_id] then
                goods_list[item_id] = module_fun.get_goods_num(character, item_id)
            end
            if goods_list[item_id] < item_num then
                flag = false
                break
            end
            goods_list[item_id] = goods_list[item_id] - item_num
        end
        if not flag then
            break
        end
        num = lv - cur_lv
    end
    return num
end

function THIS.skill_card_upgrade(character, args)
    local d = _M.assert_get(character)
    local sid = args.sid 
    local num = args.num
    if not sid then
        return STD_ERR.COMMON_PARM_ERR  -- 参数异常
    end

    if num < 0 or num > MAX_UPGRADE then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local data = d.list[sid]
    local level = data.lv
    if not data then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local card_conf_list = assert(asset.CardSkillConfig_proto, "CardSkillConfig_proto")
    local card_conf = card_conf_list[data.id]
    if not card_conf then
        logger_trace("CardSkillConfig_proto:%d", data.id)
    end

    local quality_conf_list = assert(asset.RoleQualityConfig_proto, "RoleQualityConfig_proto")
    local quality_conf = quality_conf_list[card_conf.quality]
    if not quality_conf then
        logger_trace("RoleQuatityConfig_proto:%d", card_conf.quality)
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if num == 0 then
        num = get_max_upgrade(character, level, math_min(quality_conf.maxLv, character.level))
        if num <= 0 then
            return STD_ERR.SKILL_CARD_MAX_LEVEL -- 超过最大等级
        end
    end

    if level+num > quality_conf.maxLv or level+num > character.level then
        return STD_ERR.SKILL_CARD_MAX_LEVEL -- 超过最大等级
    end

    local level_conf_list = assert(asset.CardLevelConfig_proto, "CardLevelConfig_proto")
    local pay_list = {}
    for i = 1, num do 
        local conf = level_conf_list[level+i]
        if not conf then
            logger_trace("CardLevelConfig_proto:%d", level+i)
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        for j = 1, #conf.goods do
            local item_id = conf.goods[j]
            local item_num = conf.goodNum[j]
            if item_id and item_num then
                table.insert(pay_list, {id = item_id, num = item_num})
            end
        end
    end

    local receipt = {
        module="skill_card",
        brief="卡牌技能升级",
        context=string.format("卡牌技能sid%d, 从%d升级到%d", sid, level, level+num),
        notify={
            flags="skill_card_upgrade"
        },
        detail=pay_list
    }
    local errno = payment(character, receipt)
    if errno ~=0 then
        return errno
    end
    local pre_lv = data.lv
    data.lv = data.lv+num
    _M.persist(character)
    character.dispatch("skill_card_upgrade", data.id, pre_lv, data.lv)
    return 0, {data = data}
end

function THIS.skill_card_upgrade_star(character, args)
    local d = _M.assert_get(character)
    local sid = args.sid
    if not sid then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local data = d.list[sid]
    if not data then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local card_conf_list = assert(asset.CardSkillConfig_proto, "CardSkillConfig_proto")
    local card_conf = card_conf_list[data.id]
    if not card_conf then
        logger_trace("CardSkillConfig_proto:%d", data.id)
    end

    local cur_quality = card_conf.quality
    local max_quality = asset.DataConfig_proto[5].data2
    if cur_quality >= max_quality then
        return 3 -- 超过最大突破等级
    end

    local new_conf = card_conf_list[data.id+1]
    if not new_conf then
        logger_trace("CardSkillConfig_proto:%d", data.id+1)
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if new_conf.quality ~= cur_quality+1 then
        logger_trace("CardSkillConfig_proto:%d", data.id+1)
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    local pay_list = {{id = new_conf.debris, num = new_conf.consume}}
    local receipt = {
        module="skill_card",
        brief="卡牌技能突破",
        context=string.format("卡牌技能突破:%d=>%d", data.id, data.id+1),
        notify={
            flags="skill_card_upgrade_star"
        },
        detail=pay_list
    }
    local errno = payment(character, receipt)
    if errno ~= 0 then
        return errno
    end
    d.list[sid].id = d.list[sid].id+1
    _M.persist(character)
    character.dispatch("skill_card_upstart", data.id, cur_quality, cur_quality+1)
    return 0, {data = d.list[sid]}
end

function REQUEST.skill_card_get_data(character, args)
    return func_ret("skill_card_get_data", character, args)
end

function REQUEST.skill_card_upgrade(character, args)
    return func_ret("skill_card_upgrade", character, args)
end

function REQUEST.skill_card_upgrade_star(character, args)
    return func_ret("skill_card_upgrade_star", character, args)
end

return MODULE